/*+------------------------------------------------------------------
  main implementation file

  Clifford M. Bryant, Jr.
  Adapted by Emmanuel Agu for CS 563, Spring 2005
*/


// disable warning C4786: symbol greater than 255 character,
// okay to ignore (See MSDN.)
#pragma warning(disable: 4786) 

#include "windows.h"
#include <vector>
#include "ply.h"
#include "ppm.h"

// Global variables
CPly g_ply;
vector<string> g_vecFilenames;
int g_iModel = 0;
string g_strPlyDir(".\\ply\\");
string g_strWindowTitle("CS 563: Project 2!");

// Mouse interface
int g_iLeftButtonDown = 0;
int g_iShiftLeftButtonDown = 0;           
int g_iOldX, g_iOldY, g_iNewX, g_iNewY;

// These values are changed in the motion() callback, and
// they are then used to set up viewing and some modeling parameters.
double g_dElevation = 0;
double g_dAzimuth = 0;
double g_dDist = 8;

// Window dimensions
int g_iww = 640, g_iwh = 480;


void init()
{
	// Compile a list of the ply files.
	WIN32_FIND_DATA ffd;
	HANDLE h = ::FindFirstFile((g_strPlyDir + "*.ply").c_str(), &ffd);
	g_vecFilenames.push_back(string(ffd.cFileName));
	while (::FindNextFile(h, &ffd))
		g_vecFilenames.push_back(string(ffd.cFileName));
	::FindClose(h);

	// Read in the first model.
	g_ply.read((g_strPlyDir + g_vecFilenames[g_iModel]).c_str());
	glutSetWindowTitle((g_strWindowTitle + " - " + g_vecFilenames[g_iModel]).c_str());

	// Calculate the averaged normals at the vertices.
	g_ply.calcNormals();

	// Set the initial window height.
	g_iwh = g_iww / CVertex::getAspect();
	g_dDist = -CVertex::getDist();

    // Enable depth test
    glEnable(GL_DEPTH_TEST);
	glShadeModel(GL_SMOOTH);
}



void display (void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();


    // Set the camera coordinates.
    gluLookAt(0, 0, g_dDist, 
              0, 0, 0, 
              0, 1, 0);

	// Translate the model down the negative z-axis.
	glTranslated(0, 0, CVertex::getDist());

	// Perform the model rotations according to the mouse movements.
    glRotatef(g_dElevation, 1, 0, 0);
    glRotatef(g_dAzimuth, 0, 1, 0);

	// Center the model at the origin.
	CVertex::centerModel();

/*
 * TODO: set up shader.
 */

	g_ply.draw();

/*
 * TODO: shut down shader.
 */

    glutSwapBuffers();
}



void resize (int w, int h)
{
    g_iww = w;
    g_iwh = h;

    // Prevent a divide by zero
    if (h == 0) h = 1;
	
    // Set Viewport to window dimensions
    glViewport(0, 0, w, h);

    // Set viewing frustum
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(CVertex::getFovy(), static_cast<double>(w) / h,
		CVertex::getNear(), CVertex::getFar());
}



void mouse (int button, int state, int x, int y)
{
    g_iNewX = x; 
    g_iNewY = y;

    // Left mouse button pressed
    if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN)
    {
        g_iLeftButtonDown = 1;
        g_iShiftLeftButtonDown = (glutGetModifiers() == GLUT_ACTIVE_SHIFT);
    }
    else
    {
        g_iShiftLeftButtonDown = g_iLeftButtonDown = 0;
    }
}



void motion (int x, int y)
{
    double dRelX, dRelY;

    g_iOldX = g_iNewX; 
    g_iOldY = g_iNewY;
    g_iNewX = x;    
    g_iNewY = y;

    if (g_iShiftLeftButtonDown) 
    {
        // Change distance
		dRelY = (g_iNewY - g_iOldY);
        g_dDist += (dRelY * CVertex::getDepth() / 10);

    }
	else if (g_iLeftButtonDown) 
    { 
		// Change Rotation
	    dRelX = (g_iNewX - g_iOldX);
		dRelY = (g_iNewY - g_iOldY);
        g_dAzimuth += dRelX;
        g_dElevation += dRelY;
    }

    glutPostRedisplay();
}



void keyboard (unsigned char c, int x, int y)
{
    switch (c) 
    {
    case 27:
    case 'q':
        exit(0);
        break;
	case 'n':
	case 'N':
		g_iModel++;
		if (g_iModel > g_vecFilenames.size()) g_iModel = 0;

		// Read in the first model.
		g_ply.read((g_strPlyDir + g_vecFilenames[g_iModel]).c_str());
		glutSetWindowTitle((g_strWindowTitle + " - " + g_vecFilenames[g_iModel]).c_str());

		// Calculate the averaged normals at the vertices.
		g_ply.calcNormals();

		// Set the initial window height.
		g_iwh = g_iww / CVertex::getAspect();
		g_dDist = CVertex::getDepth();

		glutReshapeWindow(g_iww, g_iwh);
		break;
	case 'p':
	case 'P':
		g_iModel--;
		if (g_iModel < 0) g_iModel = g_vecFilenames.size() - 1;

		// Read in the first model.
		g_ply.read((g_strPlyDir + g_vecFilenames[g_iModel]).c_str());
		glutSetWindowTitle((g_strWindowTitle + " - " + g_vecFilenames[g_iModel]).c_str());

		// Calculate the averaged normals at the vertices.
		g_ply.calcNormals();

		// Set the initial window height.
		g_iwh = g_iww / CVertex::getAspect();
		g_dDist = CVertex::getDepth();

		glutReshapeWindow(g_iww, g_iwh);
		break;
    }
}



void selectShader(int whichShader)
{
/*
 * TODO: Implement me!
 */
}



void create_menus()
{
    // Create the main menu.
    glutCreateMenu (selectShader);
    glutAddMenuEntry("Enhanced Phong",          0);
    glutAddMenuEntry("Eye-space lighting 1",    1);
    glutAddMenuEntry("Eye-space lighting 2",    2);
    glutAddMenuEntry("Eye-space lighting 3",    3);
    glutAddMenuEntry("Oren Nayar",              4);
    glutAddMenuEntry("Env Mapping",             5);
    
    glutAttachMenu(GLUT_RIGHT_BUTTON);
}



int main(int argc, char* argv[])
{
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
    glutInitWindowSize(g_iww, g_iwh);
    glutInitWindowPosition(180, 100);
    glutCreateWindow(g_strWindowTitle.c_str());

    glutDisplayFunc(display);
    glutReshapeFunc(resize);
    glutMouseFunc(mouse);
    glutMotionFunc(motion);
    glutKeyboardFunc(keyboard);

    init();

	create_menus();

    glutMainLoop();
    return 0;
}
