/*+------------------------------------------------------------------
  ply.h - ply file header file

  Clifford M. Bryant, Jr.
*/


#include <string>
#include <math.h>
#include <GL/glut.h>

using namespace std;

#define RADDEG 57.29578
#define DEGRAD 0.0174533

// forward declaration
class CPly;

class CVertex
{
public:
	CVertex() { n[0] = n[1] = n[2] = 0; }
	// Increase the small fov's and decrease the large fov's.
	static double getFovy()
		{ double angle = 1.5 * RADDEG * atan( (max[1] - min[1]) / fabs(getDist()) );
		  return ( (angle < 25) ? pow(25 / angle, .25) * angle :
			       (angle > 45) ? sqrt(45 / angle) * angle : angle ); }
	static double getAspect() { return ( max[0] - min[0] ) / ( max[1] - min[1] ); }
	static double getDepth() { return ( max[2] - min[2] ); }
	static double getNear() { return .5 * getDepth(); }
	static double getFar() { return 10 * getDepth(); }
	static double getDist() { return -2.5 * getDepth(); }
	static void centerModel() { glTranslated( -(min[0] + max[0]) / 2,
		-(min[1] + max[1]) / 2, -(min[2] + max[2]) / 2); }
	static void reset() { min[0] = min[1] = min[2] = 1.e38; max[0] = max[1] = max[2] = -1.e38; }
	static double getLight0X() { return (5 * (max[0] - min[0])); }
	static double getLight0Y() { return (2 * (max[1] - min[1])); }
	static double getLight1Y() { return (10 * (max[1] - min[1])); }
	static double getLight2Y() { return (8 * (max[1] - min[1])); }
	static double getLight2Z() { return (8 * (max[2] - min[2])); }
	static double getLight2ZCenter() { return (max[2] + min[2]) / 2; }
	void sumNormal(double * _n);
	void normalize();
	static double getXRange() { return (max[0] - min[0]); }
	static double getYRange() { return (max[1] - min[1]); }

private:
	void calcStatistics();
	double c[3];	// cordinate vector
	double n[3];	// normal vector
	static double min[3];
	static double max[3];
	friend istream & operator>>(istream & is, CVertex & v);
	friend class CTriangle;
	friend class CPly;  // for printing out dimension statistics
};

class CTriangle
{
public:
	static void setVertices(CVertex * pVertices) { m_pVertices = pVertices; }
	void draw();
	void calcNormal();
	void sumNormals();
	static void setShading(int iShading) { m_iShading = iShading; }

private:
	int v[3];		// triangle vertices
	double n[3];	// normal vector
	static CVertex * m_pVertices;
	static int m_iShading;
	friend istream & operator>>(istream & is, CTriangle & t);
};

class CPly
{
public:
	CPly() : m_pVertices(NULL), m_pTriangles(NULL) {}
	~CPly() { if (m_pVertices) delete [] m_pVertices;
		if (m_pTriangles) delete [] m_pTriangles; }
	int read(const string strFilename);
	void draw();
	void calcNormals();

private:
	int m_iVertices;
	CVertex * m_pVertices;
	int m_iTriangles;
	CTriangle * m_pTriangles;
};
