/*+------------------------------------------------------------------
  Portable Pixmap class implementation file

  Clifford M. Bryant, Jr.
*/

#include <string.h>
#include <stdio.h>
#include "ppm.h"
 
CPPM::CPPM()
	: m_iWidth(0), m_iHeight(0), m_iColorMax(0), m_prgbImage(NULL)
{
}

CPPM::~CPPM()
{
	if (m_prgbImage) delete m_prgbImage;
}

void CPPM::ppmRead(const char * szFileName)
{
	FILE * fin;
	char buff[BUFFSIZE];
	int i, j;

	// Was the file open successful?
	if ((fin = fopen(szFileName, "rb")) == NULL) return;

	// Check for the "magic number."
	fgets(buff, BUFFSIZE, fin);
	if (strcmp(buff, "P6\n")) return;

	// Read the image size.
	ppmReadHeaderLine(fin, buff);
	sscanf(buff, "%d %d", &m_iWidth, &m_iHeight);

	// Read the maximum value per color channel.
	ppmReadHeaderLine(fin, buff);
	sscanf(buff, "%d", &m_iColorMax);

	// Read in the image.
	if (m_prgbImage) delete m_prgbImage;
	m_prgbImage = new SRGB[m_iWidth * m_iHeight];
	int iCount = fread(reinterpret_cast<unsigned char *>(m_prgbImage),
		m_iWidth * m_iHeight * sizeof(SRGB), 1, fin);

	// Close the image file.
	fclose(fin);

	// Flip the image top to bottom.
	SRGB rgb;
	for (i = 0; i < m_iHeight / 2; i++)
	{
		for (j = 0; j < m_iWidth; j++)
		{
			rgb = m_prgbImage[i * m_iWidth + j];
			m_prgbImage[i * m_iWidth + j] = m_prgbImage[(m_iHeight - i - 1) * m_iWidth + j];
			m_prgbImage[(m_iHeight - i - 1) * m_iWidth + j] = rgb;
		}
	}
}

void CPPM::ppmReadHeaderLine(FILE * fin, char * buff)
{
	do {
		fgets(buff, BUFFSIZE, fin);
	} while (buff[0] == '#');
}
